/******************************************************************************
 *
 * LF.net ipcount $Id$
 *
 * ipcount snifft auf einem Netzwerk Interface die Pakete, und sortiert die
 * Anzahl der Pakete und deren Groessen-Summe in eine Matrix mit src:dst 
 * als Key ein. ipcount wird zu Accounting Zwecken verwendet.
 *
 * 
 * mah, Thu Jul 23 09:58:15 CEST 2009
 * - initiale Version, Rewrite der Version 3
 *
 ******************************************************************************/

#include "ipv4.h"
#include "ipcount_error.h"

#include <iostream>
#include <string>
#include <map>

#include <inttypes.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>

using namespace std;

// Compare Funktion der Traffic Matrix; diese ist noetig, da die Std-C++
// Lib natuerlich unseren Typ ipv4_hashkey nicht kennt. Damit die map
// funktioniert, muss sie einen Vergleich durchfuehren koennen.
bool ipv4_compare_function::operator()(const ipv4_hashkey &k1, 
		const ipv4_hashkey &k2) const
{
	return (
			(k1.src < k2.src) ||
			((k1.src == k2.src) && (k1.dst < k2.dst)));
}

ipv4_hash iphash;

// Packet Handler; wird von pcap_iface aufgerufen, wenn ein Paket zu
// analysieren ist
void ipv4_packet_handler(const struct pcap_pkthdr* pkthdr,
		const uint8_t* packet, pcap_counters *counters)
{

	// Gesamt-Laenge des Frames (Ethernet)
	u_int length = pkthdr->len;

	u_int hlen,off,version;
	u_int len;

	const ipv4_header* ip;
	// Ptr auf den IP Header setzen
	ip = (ipv4_header*)(packet + sizeof(struct ether_header));

	// von der Gesamt-Laenge den Teil des Ethernet Headers abziehen
	length -= sizeof(struct ether_header);

	// pruefen, ob Paket verstuemmelten IP Header hat
	if (length < sizeof(ipv4_header))
	{
		counters->truncated_ip++;
		return;
	}

	// len auf dem IP Header auslesen
	len     = ntohs(ip->ip_len);

	// IP Header Length extrahieren
	hlen    = IP_HL(ip);

	// IP Version extrahieren
	version = IP_V(ip);

	// Version pruefen
	if (version != 4)
	{
		counters->unknown_version++;
		return;
	}

	// Header Laenge pruefen
	if (hlen < 5 )
	{
		counters->bad_header_len++;
		return;
	}

	// Falls IP-Gesamt-Laenge kleiner als im IP-Header angegebene Laenge
	if (length < len)
	{
		counters->truncated_packet++;
		// hier weiter machen, das koennen wir accounten
	}

	// Pruefen, ob wir das erste Fragment haben, denn nur das wird gezaehlt,
	// da im Header die Laenge ueber alle Fragmente steht
	off = ntohs(ip->ip_off);
	if ((off & 0x1fff) == 0 )
	{
		// Hash Key erzeugen
		ipv4_hashkey k;
		k.src = ip->ip_src.s_addr;
		k.dst = ip->ip_dst.s_addr;

		// Hash Daten holen
		ipcount_data v = iphash[k];

		// Daten updaten
		v.bytes += len;
		v.packets += 1;

		// Hash Daten wieder schreiben
		iphash[k] = v;
	}
	else
		counters->fragments++;
}

void ipv4_write_iphash(const string &filename)
{
	// File mit Append Mode oeffnen
	FILE *f = fopen (filename.c_str(),"a");
	if (f == 0)
		throw ipcount_error(strerror(errno));

	// ueber den Traffic Matrix Hash iterieren
	for (ipv4_hash::iterator i = iphash.begin(); i != iphash.end(); i++)
	{
		// Key und Value auslesen
		const ipv4_hashkey k = i->first;
		const ipcount_data v = i->second;

		// passende IPv4 structs fuellen
		struct in_addr src,dst;
		src.s_addr = k.src;
		dst.s_addr = k.dst;

		// von der 32bit Variante in die dotted Notation uebersetzen
		// ACHTUNG: die inet_ntoa() ist NICHT reentrant, daher wird das
		// hier hintereinander gemacht. Der Compiler erzeugt sonst ggfls
		// interleaved code.
		const string s1(inet_ntoa(src));
		const string s2(inet_ntoa(dst));

		// printf verwenden, da es schneller ist als die C++ Streams
		fprintf(f,"%s %s %ju %ju\n",s1.c_str(),s2.c_str(),
				v.bytes,
				v.packets);

	}
	fclose(f);
}

void ipv4_clear_hash()
{
	// Hash leeren
	iphash.clear();
}

void ipv4_write_stats(const string &filename, pcap_counters *counters,
		bool append )
{
	// File mit Append Mode oeffnen
	FILE *f = fopen (filename.c_str(),append ? "a" : "w");
	if (f == 0)
		throw ipcount_error(strerror(errno));

	// aktuelle Zeit besorgen
	const time_t now = time(0);
	// in struct ueberfuehren
	const struct tm *tm = localtime(&now);
	// und in einen passenden String ueberfuehren
	char s[128];
	strftime(s, sizeof(s)-1, "%s %Y%m%d-%H:%M:%S", tm);

	// alle Felder der (error)-Counters struct ausgeben
	fprintf(f,"--- %s ---\n",s);
	fprintf(f,"dropped: %ju\n", counters->capture_drop + counters->write_drop);
	fprintf(f,"dropped-while-written: %ju\n", counters->write_drop);
	fprintf(f,"dropped-while-capturing: %ju\n", counters->capture_drop);
	fprintf(f,"dropped-last-5min: %ju\n", counters->deltadrops);
	fprintf(f,"pcap_next_ex_errors: %ju\n", counters->pcap_next_ex_errors);
	fprintf(f,"truncated-ip-headers: %ju\n", counters->truncated_ip);
	fprintf(f,"unknown-ip-version: %ju\n", counters->unknown_version);
	fprintf(f,"bad-header-len: %ju\n", counters->bad_header_len);
	fprintf(f,"truncated-packets: %ju\n", counters->truncated_packet);
	fprintf(f,"recvd-packets: %ju\n", counters->recvd);
	fprintf(f,"interface-dropped-packets: %ju\n", counters->ifdrop);
	fprintf(f,"fragments: %ju\n", counters->fragments);
	fprintf(f,"hash-entries: %ju\n", (uint64_t)iphash.size());
	fprintf(f,"sum-dropped-percent: %2.2f %%\n", 
			(100.0 * (counters->capture_drop + counters->write_drop)
			 / counters->recvd));
	fprintf(f,"\n");

	fclose(f);
}

