/******************************************************************************
 *
 * LF.net ipcount $Id$
 *
 * ipcount snifft auf einem Netzwerk Interface die Pakete, und sortiert die
 * Anzahl der Pakete und deren Groessen-Summe in eine Matrix mit src:dst 
 * als Key ein. ipcount wird zu Accounting Zwecken verwendet.
 *
 * 
 * mah, Thu Jul 23 09:58:15 CEST 2009
 * - initiale Version, Rewrite der Version 3
 *
 ******************************************************************************/

#ifndef IPV4_H
#define IPV4_H

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h>

#include <map>

#include "pcap_iface.h"

using namespace std;

// IPv4 Header; wurde von tcpdump uebernommen, da dieser Header scheinbar
// nirgends in den Std-includes zu finden ist
typedef struct
{
	uint8_t	ip_vhl;							// header length, version 
#define IP_V(ip)	(((ip)->ip_vhl & 0xf0) >> 4)
#define IP_HL(ip)	((ip)->ip_vhl & 0x0f)
	uint8_t	ip_tos;							// type of service 
	uint16_t ip_len;						// total length 
	uint16_t ip_id;							// identification 
	uint16_t ip_off;						// fragment offset field 
#define	IP_DF 0x4000						// dont fragment flag 
#define	IP_MF 0x2000						// more fragments flag 
#define	IP_OFFMASK 0x1fff					// mask for fragmenting bits 
	uint8_t	ip_ttl;							// time to live 
	uint8_t	ip_p;							// protocol 
	uint16_t ip_sum;						// checksum 
	struct in_addr ip_src,ip_dst;			// source and dest address 
} ipv4_header;

// Hashkey Typ der Traffic-Matrix
typedef struct
{
	uint32_t src;
	uint32_t dst;
} ipv4_hashkey;

// Hash Value Typ der Traffic-Matrix
typedef struct
{
	uint64_t packets;
	uint64_t bytes;
} ipcount_data;

// Compare Funktion der Traffic Matrix; diese ist noetig, da die Std-C++
// Lib natuerlich unseren Typ ipv4_hashkey nicht kennt. Damit die map
// funktioniert, muss sie einen Vergleich durchfuehren koennen.
struct ipv4_compare_function
{
	bool operator()(const ipv4_hashkey &k1, const ipv4_hashkey &k2) const;
};

// Typ des Traffic Matrix Hashes fuer IPv4
typedef map<ipv4_hashkey, ipcount_data, ipv4_compare_function> ipv4_hash;

// Packet Handler; wird von pcap_iface aufgerufen, wenn ein Paket zu
// analysieren ist
void ipv4_packet_handler(const struct pcap_pkthdr* pkthdr,
		const uint8_t* packet, pcap_counters *counters);

// schreibt den Hash in ein traffic File
void ipv4_write_iphash(const string &filename);

// loescht alle Hash Eintraege
void ipv4_clear_hash();

// schreibt die Stats in ein stats File
void ipv4_write_stats(const string &filename, pcap_counters *counters,
		bool append = true);

#endif
