/******************************************************************************
 *
 * LF.net ipcount $Id$
 *
 * ipcount snifft auf einem Netzwerk Interface die Pakete, und sortiert die
 * Anzahl der Pakete und deren Groessen-Summe in eine Matrix mit src:dst 
 * als Key ein. ipcount wird zu Accounting Zwecken verwendet.
 *
 * 
 * mah, Thu Jul 23 09:58:15 CEST 2009
 * - initiale Version, Rewrite der Version 3
 *
 ******************************************************************************/

#include "pcap_iface.h"
#include <string>
#include <iostream>
#include "ipcount_error.h"

using namespace std;

pcap_iface::pcap_iface(const string &device)
{
	char errbuf[PCAP_ERRBUF_SIZE];

	// Interface in den promiscous Mode setzen und oeffnen; es wird ein
	// Capture Timeout von 1000ms verwendet. Die blocking-Calls dauern dann
	// max 1000s.
	m_handle = pcap_open_live(device.c_str(), pcap_iface::snaplen, 
			1, 1000, errbuf);
	if (m_handle == 0)
	{
		throw ipcount_error(errbuf);
	}

	// stats initialisieren:
	m_counters.pcap_next_ex_errors = 0;
	m_counters.truncated_ip = 0;
	m_counters.unknown_version = 0;
	m_counters.bad_header_len = 0;
	m_counters.truncated_packet = 0;
	m_counters.capture_drop = 0;
	m_counters.write_drop = 0;
	m_counters.recvd = 0;
	m_counters.last_recvd = 0;
	m_counters.ifdrop = 0;
	m_counters.fragments = 0;
	m_last_drops = 0;
}

pcap_iface::~pcap_iface()
{
	pcap_close(m_handle);
}

void pcap_iface::set_filter(const string &filter)
{
	char errbuf[PCAP_ERRBUF_SIZE];

	int res = pcap_compile(m_handle, &m_bpfprogram, filter.c_str(),
			0, m_netp);
	if (res == -1)
	{
		const string err(pcap_geterr(m_handle));
		throw ipcount_error(errbuf);
	}

	res = pcap_setfilter(m_handle, &m_bpfprogram);
	if (res == -1)
	{
		const string err(pcap_geterr(m_handle));
		throw ipcount_error(err);
	}
}

void pcap_iface::capture_packet(pcap_packet_handler handler)
{
	struct pcap_pkthdr *pkt_header;
	const uint8_t *pkt_data;

	// Paket blocking lesen; die Funktion kehrt nach spaetestens 1000ms
	// zurueck, oder sofort, wenn ein Paket zu lesen ist
	//
	// Bzgl. Packetloss lesenswert:
	// http://www.mail-archive.com/ntop@unipi.it/msg10904.html
	//
	int res = pcap_next_ex(m_handle, &pkt_header, &pkt_data);

	// Wenn "the packet was read without problems" oder
	// "packets are being read from a live capture, and the timeout expired"
	if ((res == 0) || (res == 1))
	{
		// Paket je nach IP Version behandeln:
		handler(pkt_header, pkt_data, &m_counters);
	}
	else
		m_counters.pcap_next_ex_errors++;
}

pcap_counters *pcap_iface::counters()
{
	return &m_counters;
}

uint64_t pcap_iface::get_delta_drops()
{
	struct pcap_stat stats;

	// pcap Stats auslesen
	int res = pcap_stats(m_handle, &stats);
	if (res != 0)
	{
		const string err(pcap_geterr(m_handle));
		throw ipcount_error(err);
	}

	// Differenz aus letzem und aktuellen Counter Stand ermitteln
	int64_t deltadrop = stats.ps_drop - m_last_drops;

	// Ueberlauf behandeln
	if (deltadrop < 0)
		deltadrop += (uint64_t)1<<32;

	// und letzten Counter Stand updaten
	m_last_drops = stats.ps_drop;

	// Anzahl der empfangenen Pakete; Ueberlauf behandeln
	if (stats.ps_recv < m_counters.last_recvd)
	{
		m_counters.recvd += stats.ps_recv - 
			m_counters.last_recvd + ((uint64_t)1<<32);
	}
	else
	{
		m_counters.recvd += stats.ps_recv - m_counters.last_recvd;
	}

	m_counters.last_recvd = stats.ps_recv;

	// und Anzahl der vom Interface verworfenen Pakete
	m_counters.ifdrop = stats.ps_ifdrop;

	return deltadrop;
}


