/******************************************************************************
 *
 * LF.net ipcount $Id$
 *
 * ipcount snifft auf einem Netzwerk Interface die Pakete, und sortiert die
 * Anzahl der Pakete und deren Groessen-Summe in eine Matrix mit src:dst 
 * als Key ein. ipcount wird zu Accounting Zwecken verwendet.
 *
 * 
 * mah, Thu Jul 23 09:58:15 CEST 2009
 * - initiale Version, Rewrite der Version 3
 *
 ******************************************************************************/

#ifndef PCAP_IFACE_H
#define PCAP_IFACE_H

#include <string>
#include <pcap.h>

using namespace std;

// Sammelstelle fuer alle Arten von Counters; die last_* Variablen halten
// den jeweils letzten Messwert, um einen 32bit Ueberlauf seitens der
// libpcap zu erkennen und zu behandeln
typedef struct
{
	// bei Fehlern der pcap_next_ex() Funktion
	uint64_t pcap_next_ex_errors;

	// bei verstuemmeltem IP Header
	uint64_t truncated_ip;

	// bei unbekannter IP Version
	uint64_t unknown_version;

	// bei zu kurzem IP Header (Header Laengen Feld)
	uint64_t bad_header_len;

	// Falls IP-Gesamt-Laenge kleiner als im IP-Header angegebene Laenge
	uint64_t truncated_packet;

	// verlorene Pakete waehrend dem Capturen
	uint64_t capture_drop;

	// verlorene Pakete seit dem letzen Messen (vor 5min)
	uint64_t deltadrops;

	// verlorene Pakete waehrend dem Traffic-Dumpen
	uint64_t write_drop;

	// Anzahl der emfpangenen Pakete
	uint64_t recvd;
	int64_t last_recvd;

	// Anzahl der vom Interface verworfenen Pakete
	uint64_t ifdrop;

	// Anzahl der IP Fragmente
	uint64_t fragments;
} pcap_counters;

// Typ der Packet-Handler Funktion
typedef void (*pcap_packet_handler)(const struct pcap_pkthdr* pkthdr,
		const uint8_t* packet, pcap_counters *counters);

// Bildet ein Interface mit der libpcap ab; eine Klasse wurde daraus gemacht,
// weil spaeter moeglicherweise auf mehren Interfaces gelauscht werden soll
class pcap_iface
{
	public:
		// Anzahl der Bytes, die die libpcap vom Paket holen soll; uns
		// interessiert nur der IP-Header
		enum { snaplen = 68 };

		pcap_iface(const string &device);
		~pcap_iface();

		// compiliert einen neuen pcap Filter und aktiviert ihn
		void set_filter(const string &filter);

		// wird von der Main Loop aufgerufen, wenn ein Paket (blocking) 
		// von der libpcap geholt werden soll
		void capture_packet(pcap_packet_handler handler);

		// liefert einen Zeiger auf die Counters struct
		pcap_counters *counters();

		// liefert die Verluste seit dem letzten Aufruf dieser Funktion;
		// dabei werden auch die anderen Felder in der pcap_counters struct
		// aktualisiert
		uint64_t get_delta_drops();

	private:
		// das Handle der libpcap
		pcap_t *m_handle;

		// das compilierte Filterprogramm
		struct bpf_program m_bpfprogram;

		// Netz + Maske des Interfaces, auf dem gelauscht wird
		bpf_u_int32 m_netp;
		bpf_u_int32 m_maskp;

		// die Counter struct
		pcap_counters m_counters;

		// der letzte Drop-Counter Stand seit dem letzten Aufruf der
		// get_delta_drops() Funktion
		uint64_t m_last_drops;

};

#endif
